/*******************************************************************************
 * Function: void Lcd_Port (char a)
 * Description: LCD Setup 4-bit data bus (sub-routine)
 ******************************************************************************/

void Lcd_Port(char a) {
    D4 = (a & 1) ? 1 : 0;
    D5 = (a & 2) ? 1 : 0;
    D6 = (a & 4) ? 1 : 0;
    D7 = (a & 8) ? 1 : 0;
}

/*******************************************************************************
 * Function: void Lcd_Cmd (char a)
 * Description: Send LCD command (nibble, sub-routine)
 ******************************************************************************/

void Lcd_Cmd(char a) {
    RS = 0;
    __delay_us(1);

    Lcd_Port(a);
    EN = 0;
    __delay_us(1);
    EN = 1;
    __delay_us(1);
    EN = 0;
    __delay_us(100);
}

/*******************************************************************************
 * Function: void Lcd_Clear()
 * Description: Clears the LCD
 ******************************************************************************/

void Lcd_Clear() {
    Lcd_Cmd(0);
    Lcd_Cmd(1);
    __delay_us(1520); // !!! Need delay after clean commands
}

/*******************************************************************************
 * Function: void Lcd_Home()
 * Description: Return Home Cursor
 ******************************************************************************/

void Lcd_Home() {
    Lcd_Cmd(0);
    Lcd_Cmd(2);
    __delay_us(1520); // !!! Need delay after clean commands
}

/*******************************************************************************
 * Function: void Lcd_Set_Cursor(char a, char b)
 * Description: Sets the LCD cursor position
 ******************************************************************************/

void Lcd_Set_Cursor(char a, char b) {
    unsigned char x, z, y;
    if (a == 1) {
        x = 0x80 + b - 1;
        z = x >> 4;
        y = x & 0x0F;
        Lcd_Cmd(z);
        Lcd_Cmd(y);
    } else if (a == 2) {
        x = 0xC0 + b - 1;
        z = x >> 4;
        y = x & 0x0F;
        Lcd_Cmd(z);
        Lcd_Cmd(y);
    }
}

/*******************************************************************************
 * Function: void Lcd_Init()
 * Description: Initialises the LCD (16*2)
 ******************************************************************************/

void Lcd_Init() {

    RS = 0;
    EN = 0;
    __delay_us(50000);

    Lcd_Cmd(0x03);
    __delay_us(4500);
    Lcd_Cmd(0x03);
    __delay_us(4500);
    Lcd_Cmd(0x03);
    __delay_us(150);

    Lcd_Cmd(0x02);

    Lcd_Cmd(0x02); // [0b001LNF--] L = 1 - 4-bit
    Lcd_Cmd(0x08); // N = 1 - 2-line, F = 0 - 5*8-dot
    __delay_us(37);

    Lcd_Cmd(0x00); // [0b00001DCB] D = 1 - display on
    Lcd_Cmd(0x0C); // C = 0 - cursor off, B = 0 - blinking cursor off
    __delay_us(37);

    //    Lcd_Cmd(0x00); // Clear display
    //    Lcd_Cmd(0x01);
    //    __delay_us(1520); // !!! Need delay after clean commands

    Lcd_Cmd(0x00); // [0b000001IS] I = 1 - Increment by 1, S = 0 - No shift
    Lcd_Cmd(0x06);
    __delay_us(37);

}

/*******************************************************************************
 * Function: void Lcd_Write_Char (char a)
 * Description: Writes a character to the LCD
 ******************************************************************************/

void Lcd_Write_Char(char a) {
    unsigned char l, h;

    l = a & 0x0F;
    h = a >> 4;

    RS = 1;
    __delay_us(1);

    Lcd_Port(h);
    EN = 0;
    __delay_us(1);
    EN = 1;
    __delay_us(1);
    EN = 0;
    __delay_us(100);

    Lcd_Port(l);
    EN = 0;
    __delay_us(1);
    EN = 1;
    __delay_us(1);
    EN = 0;
    __delay_us(100);
}

/*******************************************************************************
 * Function: void Lcd_Write_String (const char *a)
 * Description: Writes a string to the LCD
 ******************************************************************************/

void Lcd_Write_String(char* a) {
    unsigned char i;
    for (i = 0; a[i] != '\0'; i++)
        Lcd_Write_Char(a[i]);
}

/*******************************************************************************
 * Function: void Lcd_Shift_Right()
 * Description: Shifts text on the LCD right
 ******************************************************************************/

void Lcd_Shift_Right() {
    Lcd_Cmd(0x01);
    Lcd_Cmd(0x0C);
    __delay_us(37);
}

/*******************************************************************************
 * Function: void Lcd_Shift_Left()
 * Description: Shifts text on the LCD left
 ******************************************************************************/

void Lcd_Shift_Left() {
    Lcd_Cmd(0x01);
    Lcd_Cmd(0x08);
    __delay_us(37);
}

/*******************************************************************************
 * Function: void Lcd_Write_Int(int v)
 * Description: Writes a integer value to the LCD
 ******************************************************************************/

void Lcd_Write_Int(int v) {
    unsigned char str[9];

    Lcd_Write_String(itoa(str, v, 10));    
}
