#define _XTAL_FREQ 4000000

// CONFIG PIC16F628A
#pragma config FOSC = XT        // Oscillator Selection bits (XT oscillator: Crystal/resonator on RA6/OSC2/CLKOUT and RA7/OSC1/CLKIN)
#pragma config WDTE = OFF       // Watchdog Timer Enable bit (WDT disabled)
#pragma config PWRTE = ON       // Power-up Timer Enable bit (PWRT enabled)
#pragma config MCLRE = ON       // RA5/MCLR/VPP Pin Function Select bit (RA5/MCLR/VPP pin function is MCLR)
#pragma config BOREN = OFF      // Brown-out Detect Enable bit (BOD disabled)
#pragma config LVP = OFF        // Low-Voltage Programming Enable bit (RB4/PGM pin has digital I/O function, HV on MCLR must be used for programming)
#pragma config CPD = OFF        // Data EE Memory Code Protection bit (Data memory code protection off)
#pragma config CP = OFF         // Flash Program Memory Code Protection bit (Code protection off)

#pragma warning disable 752

#define DS RA2
#define RS RA1
#define EN RA0
#define D4 RB7
#define D5 RB6
#define D6 RB5
#define D7 RB4

//XC8 Standard Include
#include <xc.h>
#include <stdlib.h>
#include <stdint.h>

//Other Includes
#include "LCD.h"

static unsigned int tick; // 10Hz buffer
static unsigned int sec; // second by start
unsigned char two; // 2sec interval flag
unsigned int ok; // last 2sec interval value

unsigned char dflag; // debug flag

unsigned int cnt; // ion-per-interval
unsigned long all; // ion-alltime
unsigned int cpm; // ion-per-last-minute
unsigned int usvh_z, usvh_p;
unsigned int usv_z, usv_p;

unsigned char i; // cycle counter

unsigned int buff[30] = {0}; // buffer of interval inside minute

char txt_space = ' ';
char txt_dot = '.';
char txt_zero = '0';

void __interrupt() isr(void) {

    if (INTF) {
        cnt++;
        INTF = 0;
    } else if (TMR1IE && TMR1IF) {
        TMR1H = 0xCF;
        TMR1L = 0x2C;
        TMR1IF = 0;
        tick = (tick < 9) ? tick++ : 0;
        sec = (tick == 9) ? sec++ : sec;
    }

}

void main(void) {

    tick = 0; // clear 10Hz buffer
    sec = 0; // clear second accumulator
    two = 0; // clear 2sec interval flag
    ok = 0; // clear last 2sec interval value
    cnt = 0; // clear counter
    all = 0; // clear counter

    TRISA &= ~0x03; // I\O port config
    TRISB &= ~0xF0; // I\O port config

    CMCON = 0x07; // disable comparator

    INTEDG = 1; // 1 - rising edge
    INTF = 0; // clear int flag
    INTE = 1; // enable int

    T1CKPS0 = 1; // prescaler 8 (1000000/8/10Hz = 1250)
    T1CKPS1 = 1;
    TMR1CS = 0; // internal clock
    TMR1H = 0xCF;
    TMR1L = 0x2C; // 65536 - 12500 = 53036 = 0xCF2C
    TMR1IF = 0; // clear int flag
    TMR1IE = 1; // timer1 int On
    TMR1ON = 1; // timer1 On

    PEIE = 1; // periferal int On
    GIE = 1; // global int On

    Lcd_Init();
    Lcd_Clear();

    // Introduction info
    if (DS) {
        Lcd_Write_String("GeigerCounter by");
        Lcd_Set_Cursor(2, 1);
        Lcd_Write_String("radiohobbystore");
        __delay_ms(1500);
        Lcd_Clear();
        Lcd_Write_String("OpenSW: 20.05xc8");
        Lcd_Set_Cursor(2, 1);
        Lcd_Write_String("for HW: 1.01");
        __delay_ms(1500);
        Lcd_Clear();
    }

    while (1) {

        dflag = DS;

        if ((sec % 2 == 0) && (ok < sec)) {
            ok = sec;
            two = 1; // Set flag - measurement interval pass ...
        }

        if (two == 1) {
            two = 0;
            buff[(sec % 60) / 2] = cnt; // Save interval result in buffer
            all += cnt;
            cnt = 0; // ready to math in next interval count ...
        }

        if (sec < 60) {
            // within the first minute the result is inaccurate
            cpm = 0;
            for (i = 1; i <= ((sec % 60) / 2); cpm += buff[i], i++);
            cpm = cpm * 30 / ((sec % 60) / 2);
        } else {
            // calculate exact result of the previous minute by history buffer data
            cpm = 0;
            for (i = 0; i < 30; cpm += buff[i], i++); // Summing buffer data
        }

        Lcd_Clear();
        //Lcd_Set_Cursor(1, 1);
        Lcd_Write_String("CPM:");
        Lcd_Write_Int(cpm); // print cpm
        if (sec < 60) Lcd_Write_Char('*'); // sign of inaccurate result

        if (dflag) {
            Lcd_Write_Char(txt_space);
            Lcd_Write_Char('/[');
            Lcd_Write_Int(cnt); // Debug - count by current interval (2sec)
            Lcd_Write_Char('/]');
            Lcd_Write_Char(txt_space);
            Lcd_Write_Int(sec); // Debug - passed seconds by start measurement
        }

        Lcd_Set_Cursor(2, 1);

        if (((sec + 1) % 8 == 0) || (sec % 8 == 0)) {
            usv_z = all / 9000; // for SBM20: 9000 Count = 1 Sv
            usv_p = (all - usv_z) / 9;
            Lcd_Write_String("uSv  :");
            Lcd_Write_Int(usv_z);
            Lcd_Write_Char(txt_dot);
            if (usv_p < 100) Lcd_Write_Char(txt_zero);
            if (usv_p < 10) Lcd_Write_Char(txt_zero);
            Lcd_Write_Int(usv_p);
        } else {
            usvh_z = cpm / 150; // for SDM20: 150 CPM = 1 uSv/h
            usvh_p = (cpm - usvh_z) / 0.15;
            Lcd_Write_String("uSv/h:");
            Lcd_Write_Int(usvh_z);
            Lcd_Write_Char(txt_dot);
            if (usvh_p < 100) Lcd_Write_Char(txt_zero);
            if (usvh_p < 10) Lcd_Write_Char(txt_zero);
            Lcd_Write_Int(usvh_p);
        }

        __delay_ms(300);

    }
    return; // of main
}